@@ -172,6 +172,35 @@ test_that("geks agrees with IndexNumR", {
   )
 })
 
+test_that("geks works with different splices", {
+  expect_equal(
+    splice_index(
+      with(dat, tornqvist_geks(price, quantity, period, product, window = 7))
+    ),
+    c(1.06202143784605, 1.1203648908828, 1.17858384039388, 1.23760852899173,
+      1.29782138422918, 1.35944797067126, 1.4190362067647, 1.48010601630221,
+      1.5422495225642, 1.60541380403309, 1.66962623911985, 1.73494499443271)
+  )
+  expect_equal(
+    splice_index(
+      with(dat, fisher_geks(price, quantity, period, product, window = 3)),
+      periods = 1
+    ),
+    c(1.05490954999787, 1.11049864213464, 1.16678403299157, 1.22378841881023, 
+      1.28153702754999, 1.34005894105164, 1.3993875342448, 1.45956101762071,
+      1.52062310501187, 1.58262383588933, 1.64562059096184, 1.70967935288461)
+  )
+  expect_equal(
+    splice_index(
+      with(dat, jevons_geks(price, quantity, period, product, window = 6)),
+      periods = 3
+    ),
+    c(1.18338442313092, 1.32033674805411, 1.43711591151299, 1.54248452241533,
+      1.64048980305452, 1.73334427991861, 1.82239254133738, 1.90851390842145,
+      1.99231538702512, 2.07423380909548, 2.15459410924166, 2.23364458259815)
+  )
+})
+
 test_that("geks works as a quantity index", {
   expect_equal(
     with(
